/*
bldc sensored driver sample

copyright (c) Davide Gironi, 2013

Released under GPLv3.
Please refer to LICENSE file for licensing information.
*/

#include <stdio.h>
#include <avr/io.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#define UART_BAUD_RATE 9600
#include "uart/uart.h"

#include "adc/adc.h"

#include "bldcsensored/bldcsensored.h"

//speed pot channel
#define POTSPEED_CHANNEL 3

//direction switch
#define BUTTONDIRECTION_DDR DDRD
#define BUTTONDIRECTION_PIN PIND
#define BUTTONDIRECTION_INPUT PD0

/*
 * main loop
 */
int main(void) {
	//init uart
	uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) );

	//init adc
	adc_init();

	//init interrupt
	sei();

	//init bldc driver
	bldcsensored_init();

	//direction and speed update interval counter
	uint16_t controlupd_interval = 0;

	//init potspeed and direction button
	uint16_t potspeed = 0;
	uint16_t potspeedold = 0;
	BUTTONDIRECTION_DDR &= ~(1<<BUTTONDIRECTION_INPUT);
	uint16_t direction = 0;
	uint16_t directionold = 0;


	//start the motor spinning
	bldcsensored_setstart();

	#if BLDCSENSORED_DEBUG == 1
	char printbuff[100];
	#endif

	//main loop
	for (;;) {
		#if BLDCSENSORED_DEBUG == 1
		if(bldcsensored_getspindirection() == 0) {
			uart_puts("motor not spinning\r\n");
		}
		#endif

		//do not check check for update every cycles
		controlupd_interval++;
		if(controlupd_interval > 400) {
			controlupd_interval = 0;

			#if BLDCSENSORED_DEBUG == 1
			if(bldcsensored_debugposerror != 0) {
				itoa(bldcsensored_debugstate, printbuff, 10); uart_puts(printbuff); uart_puts(" - ");
				itoa(bldcsensored_debugposerror, printbuff, 10); uart_puts(printbuff); uart_puts(" - ");
				itoa(bldcsensored_debugposerrorprev, printbuff, 10); uart_puts(printbuff); uart_puts("\r\n");
				bldcsensored_debugposerror = 0;
			}
			#endif

			//check for speed
			potspeed = ((int32_t)(64-10)*potspeed+(int32_t)10*adc_read(POTSPEED_CHANNEL))>>6; //read filtered pot speed
			if(potspeed != potspeedold) {
				potspeedold = potspeed;
				uint16_t potspeedn = (long)(potspeed - 0) * (long)(100 - 0) / (long)(1024 - 0) + 0;
				bldcsensored_setspeed(potspeedn);
			}

			//check for direction
			if(BUTTONDIRECTION_PIN & (1<<BUTTONDIRECTION_INPUT)) {
				direction = BLDCSENSORED_DIRECTIONCW;
			} else {
				direction = BLDCSENSORED_DIRECTIONCCW;
			}
			if(direction != directionold) { //direction changed
				#if BLDCSENSORED_DEBUG == 1
				if(direction == BLDCSENSORED_DIRECTIONCW) {
					uart_puts("direction changed to: cw\r\n");
				} else if(direction == BLDCSENSORED_DIRECTIONCCW) {
					uart_puts("direction changed to: ccw\r\n");
				}
				#endif
				directionold = direction;
				bldcsensored_setdirection(direction);
			}
		}

	}
	
	return 0;
}
